/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.yoko.bindings.corba;

import java.util.ArrayList;
import java.util.List;

import org.apache.yoko.bindings.corba.types.CorbaObjectHandler;

public class CorbaMessage {

    private String operation;
    private List<CorbaStreamable> arguments;
    private CorbaStreamable returnParam;
    private CorbaStreamable except;
    private Exception exception;

    public CorbaMessage() {
        operation = null;
        arguments = new ArrayList<CorbaStreamable>();
        returnParam = null;
        exception = null;
        except = null;
    }

    public String getOperationName() {
        return operation;
    }

    public void setOperationName(String opName) {
        operation = opName;
    }

    public void setException(Exception ex) {
        exception = ex;
    }

    public Exception getException() {
        return exception;
    }
    
    public CorbaStreamable getStreamableException() {
        return except;
    }

    public CorbaStreamable[] getStreamableArguments() {
        return arguments.toArray(new CorbaStreamable[0]);
    }

    public CorbaStreamable getStreamableReturn() {
        return returnParam;
    }

    public void addStreamableArgument(CorbaStreamable arg) {
        if (arguments == null) {
            arguments = new ArrayList<CorbaStreamable>(1);
        }

        arguments.add(arg);
    }

    public void setStreamableArguments(CorbaStreamable[] data) {
        if (arguments == null) {
            arguments = new ArrayList<CorbaStreamable>(data.length);
        }

        for (int i = 0; i < data.length; ++i) {
            arguments.set(i, data[i]);
        }
    }

    public void setStreamableArgumentValue(CorbaObjectHandler data, int idx) {
        if (idx >= arguments.size()) {
            throw new CorbaBindingException("setStreamableArgumentValue: Index out of range");
        }

        arguments.get(idx).setObject(data);
    }

    public void setStreamableArgumentValues(CorbaObjectHandler[] data) {
        for (int i = 0; i < data.length; ++i) {
            arguments.get(i).setObject(data[i]);
        }
    }

    public void setStreamableReturn(CorbaStreamable data) {
        returnParam = data;
    }

    public void setStreamableReturnValue(CorbaObjectHandler data) {
        // TODO: Handle case of the return parameter has not yet been initialized.
        returnParam.setObject(data);
    }
    
    public void setStreamableException(CorbaStreamable ex) {
        except = ex;
    }
    
    public void setStreamableExceptionValue(CorbaObjectHandler exData) {
        // TODO: Handle case of the return parameter has not yet been initialized.
        except.setObject(exData);
    }

    public String toString() {
        // TODO: Add a more complete toString
        return operation;
    }
}