/**
 *
 * Copyright 2006 The Apache Software Foundation or its licensors, as applicable.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.yoko.rmi.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.omg.CORBA.MARSHAL;

public class IDLEntityDescriptor extends ValueDescriptor {
    Method _read_method;

    Method _write_method;

    Method _type_method;

    boolean isAbstract = false;

    IDLEntityDescriptor(Class type, TypeRepository repository) {
        super(type, repository);
    }

    public String getIDLName() {
        return "org_omg_boxedIDL_" + super.getIDLName();
    }

    public void initIDL() {
        super.init();

        try {
            final Class type = getJavaClass();
            final String helperName = type.getName() + "Helper";
            final Class helperClass = javax.rmi.CORBA.Util.loadClass(
                    helperName, null, type.getClassLoader());

            java.security.AccessController
                    .doPrivileged(new java.security.PrivilegedAction() {
                        public Object run() {

                            try {
                                Method _id_method = null;
                                Method[] methods = helperClass
                                        .getDeclaredMethods();
                                for (int i = 0; i < methods.length; i++) {
                                    String name = methods[i].getName();

                                    if (name.equals("id"))
                                        _id_method = methods[i];

                                    else if (name.equals("read"))
                                        _read_method = methods[i];

                                    else if (name.equals("write"))
                                        _write_method = methods[i];

                                    else if (name.equals("type"))
                                        _type_method = methods[i];
                                }

                                // _repid = (String)
                                _id_method.invoke(null, new Object[0]);

                            } catch (InvocationTargetException ex) {
                                throw new RuntimeException(
                                        "cannot initialize: " + ex);

                            } catch (IllegalAccessException ex) {
                                throw new RuntimeException(
                                        "cannot initialize: " + ex);
                            }

                            return null;
                        }
                    });

        } catch (ClassNotFoundException ex) {
            throw new RuntimeException("cannot load IDL Helper class for "
                    + getJavaClass());
        }
    }

    /*
     * public String getRepositoryID () { return _repid; }
     */

    /** Read an instance of this value from a CDR stream */
    public Object read(org.omg.CORBA.portable.InputStream in) {
        return readValue(in);
    }

    public java.io.Serializable readValue(
            final org.omg.CORBA.portable.InputStream in,
            final java.util.Map offsetMap, final java.lang.Integer offset) {
        final java.io.Serializable value = (java.io.Serializable) readValue(in);

        offsetMap.put(offset, value);

        return value;
    }

    public Object readValue(org.omg.CORBA.portable.InputStream in) {
        if (isAbstract) {
            throw new MARSHAL("IDL Entity " + getJavaClass().getName()
                    + " is abstract");
        }

        try {
            return _read_method.invoke(null, new Object[] { in });
        } catch (InvocationTargetException ex) {
            throw new org.omg.CORBA.MARSHAL(ex.getMessage());
        } catch (IllegalAccessException ex) {
            throw new org.omg.CORBA.MARSHAL(ex.getMessage());
        }
    }

    /** Write an instance of this value to a CDR stream */
    public void write(org.omg.CORBA.portable.OutputStream out, Object val) {
        writeValue(out, val);
    }

    public void writeValue(org.omg.CORBA.portable.OutputStream out, Object val) {
        if (isAbstract) {
            throw new MARSHAL("IDL Entity " + getJavaClass().getName()
                    + " is abstract");
        }

        try {
            _write_method.invoke(null, new Object[] { out, val });
        } catch (InvocationTargetException ex) {
            throw new org.omg.CORBA.MARSHAL(ex.getMessage());
        } catch (IllegalAccessException ex) {
            throw new org.omg.CORBA.MARSHAL(ex.getMessage());
        }
    }

    org.omg.CORBA.TypeCode getTypeCode() {
        if (_type_code == null) {

            try {
                _type_code = (org.omg.CORBA.TypeCode) _type_method.invoke(null,
                        new Object[0]);
            } catch (InvocationTargetException ex) {
                throw new org.omg.CORBA.MARSHAL(ex.getMessage());
            } catch (IllegalAccessException ex) {
                throw new org.omg.CORBA.MARSHAL(ex.getMessage());
            }
        }

        return _type_code;
    }

}
