/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/

package org.apache.yoko.tools.processors.idl;

import javax.xml.namespace.QName;

import antlr.collections.AST;

import org.apache.schemas.yoko.bindings.corba.Const;
import org.apache.schemas.yoko.bindings.corba.TypeMappingType;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;
import org.apache.ws.commons.schema.XmlSchemaType;

public class ConstVisitor extends TypesVisitorBase {

    public ConstVisitor(XmlSchemaCollection xmlSchemas,
                        XmlSchema xmlSchema,
                        TypeMappingType typeMappingType) {
        super(xmlSchemas, xmlSchema, typeMappingType);
    }
    
    public void visit(AST constNode) {
        AST constTypeNode = constNode.getFirstChild();
        AST constNameNode = TypesUtils.getPrimitiveCorbaTypeNameNode(constTypeNode);
        AST constValueNode = constNameNode.getNextSibling();
        
        // build value string
        String constValue = constValueNode.toString();
        constValueNode = constValueNode.getFirstChild();
        while (constValueNode != null) {
            constValue = constValue + constValueNode.toString();
            constValueNode = constValueNode.getFirstChild();
        }
        
        QName constQName = new QName(typeMap.getTargetNamespace(),
                                     constNameNode.toString());

        // xsd::schemaType
        XmlSchemaType constSchemaType = TypesUtils.findType(schemas, schema, constTypeNode);
        
        // corba:type
        //// There is a problem with retrieving the corba:type from a schema:type
        // the schema:type -> corba:type is not one-to-one
        //CorbaTypeImpl constCorbaType = TypesUtils.findCorbaType(typeMap, constSchemaType.getQName());

        // const types are guaranteed to be primitive types
        QName constCorbaTypeName = TypesUtils.findCorbaTypeName(typeMap, constTypeNode);
        
        // corba:const
        Const corbaConst = new Const();
        corbaConst.setQName(constQName);
        corbaConst.setValue(constValue);
        corbaConst.setType(constSchemaType.getQName());
        //corbaConst.setIdltype(constCorbaType.getQName());
        corbaConst.setIdltype(constCorbaTypeName);
        
        typeMap.getStructOrExceptionOrUnion().add(corbaConst);
    }
}
