/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/

package org.apache.yoko.tools.processors.idl;

import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Operation;
import javax.wsdl.PortType;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensionRegistry;

import antlr.ASTVisitor;
import antlr.collections.AST;

import org.apache.schemas.yoko.bindings.corba.OperationType;

import org.apache.yoko.wsdl.CorbaConstants;

public class OperationVisitor implements ASTVisitor {

    private WSDLASTVisitor wsdlASTVisitor;
    private Definition definition;
    private ExtensionRegistry extReg;
    private PortType portType;
    private Binding binding;
    
    public OperationVisitor(WSDLASTVisitor visitor,
                            PortType wsdlPortType,
                            Binding wsdlBinding) {
        wsdlASTVisitor = visitor;
        definition = visitor.getDefinition();
        extReg = definition.getExtensionRegistry();
        portType = wsdlPortType;
        binding = wsdlBinding;
    }
    
    public void visit(AST node) {
        MessageVisitor msgVisitor = new MessageVisitor(wsdlASTVisitor);
        Operation op = definition.createOperation();
        op.setName(node.toString());
        op.setUndefined(false);
        portType.addOperation(op);
        BindingOperation bindingOperation = createBindingOperation(binding, op);
        msgVisitor.setOperation(op);
        msgVisitor.setBindingOperation(bindingOperation);
        AST node2 = node.getFirstChild();
        if (node2 != null && node2.getType() == IDLTokenTypes.LITERAL_oneway) {
            // oneway operations map to operations with input message
            msgVisitor.createInputMessage();
        } else {
            // normal operations map to request-response operations
            // with input and output messages
            msgVisitor.createInputMessage();
            msgVisitor.createOutputMessage();
        }
        msgVisitor.visit(node);
        node = node.getNextSibling();
    }

    private BindingOperation createBindingOperation(Binding wsdlBinding, Operation op) {
        BindingOperation bindingOperation = definition.createBindingOperation();
        OperationType operationType = null;
        try {
            operationType = (OperationType)extReg.createExtension(BindingOperation.class,
                                                                  CorbaConstants.NE_CORBA_OPERATION);
        } catch (WSDLException ex) {
            throw new RuntimeException(ex);
        }
        operationType.setName(op.getName());
        bindingOperation.addExtensibilityElement(operationType);
        bindingOperation.setOperation(op);
        bindingOperation.setName(op.getName());
        binding.addBindingOperation(bindingOperation);
        return bindingOperation;
    }

}
